@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2019 Nerian Vision GmbH
+ * Copyright (c) 2022 Nerian Vision GmbH
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -59,14 +59,26 @@ namespace GenTL {
     << "; " << DEBUG_VAL(c) << std::endl;}
 #define DEBUG4(a, b, c, d)  {debugStream << __FUNCTION__ << ": " << DEBUG_VAL(a) << "; " << DEBUG_VAL(b) \
     << "; " << DEBUG_VAL(c) << "; " << DEBUG_VAL(d) << std::endl;}
+#define DEBUG5(a, b, c, d, e)  {debugStream << __FUNCTION__ << ": " << DEBUG_VAL(a) << "; " << DEBUG_VAL(b) \
+    << "; " << DEBUG_VAL(c) << "; " << DEBUG_VAL(d) << "; " << DEBUG_VAL(e) << std::endl;}
+#define DEBUG6(a, b, c, d, e, f)  {debugStream << __FUNCTION__ << ": " << DEBUG_VAL(a) << "; " << DEBUG_VAL(b) \
+    << "; " << DEBUG_VAL(c) << "; " << DEBUG_VAL(d) << "; " << DEBUG_VAL(e) << "; " << DEBUG_VAL(f) << std::endl;}
+#define DEBUG7(a, b, c, d, e, f, g)  {debugStream << __FUNCTION__ << ": " << DEBUG_VAL(a) << "; " << DEBUG_VAL(b) \
+    << "; " << DEBUG_VAL(c) << "; " << DEBUG_VAL(d) << "; " << DEBUG_VAL(e) << "; " << DEBUG_VAL(f) \
+    << "; " << DEBUG_VAL(g) << std::endl;}
+
 
 #else
 
-#define DEBUG                 {}
-#define DEBUG1(a)             {}
-#define DEBUG2(a, b)          {}
-#define DEBUG3(a, b, c)       {}
-#define DEBUG4(a, b, c, d)    {}
+#define DEBUG                          {}
+#define DEBUG1(a)                      {}
+#define DEBUG2(a, b)                   {}
+#define DEBUG3(a, b, c)                {}
+#define DEBUG4(a, b, c, d)             {}
+#define DEBUG5(a, b, c, d, e)          {}
+#define DEBUG6(a, b, c, d, e, f)       {}
+#define DEBUG7(a, b, c, d, e, f, g)    {}
+
 
 #endif
 
@@ -75,17 +87,17 @@ namespace GenTL {
 */
 
 #define GENTL_TRY try {
-	
+
 #ifdef _WIN32
 #define GENTL_CATCH } catch (const std::exception& ex) {\
-	std::string msgString = "Exception occured: " + std::string(ex.what()); \
-	MessageBox(NULL, msgString.c_str(), "Nerian GenTL", MB_ICONERROR); \
-	return GC_ERR_ERROR;\
+    std::string msgString = "Exception occured: " + std::string(ex.what()); \
+    MessageBox(NULL, msgString.c_str(), "Nerian GenTL", MB_ICONERROR); \
+    return GC_ERR_ERROR;\
 }
 #else
 #define GENTL_CATCH } catch (const std::exception& ex) {\
-	std::cout << "Exception: " << ex.what(); \
-	return GC_ERR_ERROR;\
+    std::cout << "Exception: " << ex.what(); \
+    return GC_ERR_ERROR;\
 }
 #endif
 
@@ -109,31 +121,31 @@ inline GenTL::GC_ERROR setLastError(GenTL::GC_ERROR error) {
  */
 
 GC_API GCInitLib(void) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG;
     return setLastError(Library::initLib());
-	GENTL_CATCH;
+    GENTL_CATCH;
 }
 
 GC_API GCCloseLib(void) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG;
     return setLastError(Library::closeLib());
-	GENTL_CATCH;
+    GENTL_CATCH;
 }
 
 GC_API GCGetInfo(TL_INFO_CMD iInfoCmd, INFO_DATATYPE* piType, void* pBuffer, size_t* piSize) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG1(iInfoCmd);
     return setLastError(Library::getInfo(iInfoCmd, piType, pBuffer, piSize));
-	GENTL_CATCH;
+    GENTL_CATCH;
 }
 
 GC_API GCGetLastError(GC_ERROR* piErrorCode, char* sErrorText, size_t* piSize) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG;
     return setLastError(Library::getLastError(piErrorCode, sErrorText, piSize, lastError));
-	GENTL_CATCH;
+    GENTL_CATCH;
 }
 
 
@@ -142,103 +154,103 @@ GC_API GCGetLastError(GC_ERROR* piErrorCode, char* sErrorText, size_t* piSize) {
  */
 
 GC_API TLOpen(TL_HANDLE* phSystem) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG;
     return setLastError(System::open(reinterpret_cast<System**>(phSystem)));
-	GENTL_CATCH;
+    GENTL_CATCH;
 }
 
 GC_API TLClose(TL_HANDLE hSystem) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG1(hSystem);
-	
+
     if(!verifyHandle(hSystem, Handle::TYPE_SYSTEM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(System::close(reinterpret_cast<System*>(hSystem)));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API TLUpdateInterfaceList(TL_HANDLE hSystem, bool8_t* pbChanged, uint64_t iTimeout) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG1(hSystem);
-	
+
     if(!verifyHandle(hSystem, Handle::TYPE_SYSTEM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<System*>(hSystem)->updateInterfaceList(pbChanged, iTimeout));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API TLGetNumInterfaces(TL_HANDLE hSystem, uint32_t* piNumIfaces) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG1(hSystem);
-	
+
     if(!verifyHandle(hSystem, Handle::TYPE_SYSTEM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<System*>(hSystem)->getNumInterfaces(piNumIfaces));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API TLGetInterfaceID(TL_HANDLE hSystem, uint32_t iIndex, char* sIfaceID, size_t* piSize) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG2(hSystem, iIndex);
-	
+
     if(!verifyHandle(hSystem, Handle::TYPE_SYSTEM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<System*>(hSystem)->getInterfaceID(iIndex, sIfaceID, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API TLGetInterfaceInfo(TL_HANDLE hSystem, const char* sIfaceID, INTERFACE_INFO_CMD iInfoCmd,
         INFO_DATATYPE* piType, void* pBuffer, size_t* piSize) {
-	GENTL_TRY;		
+    GENTL_TRY;
     DEBUG3(hSystem, sIfaceID, iInfoCmd);
-	
+
     if(!verifyHandle(hSystem, Handle::TYPE_SYSTEM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<System*>(hSystem)->getInterfaceInfo(sIfaceID, iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API TLOpenInterface(TL_HANDLE hSystem, const char* sIfaceID, IF_HANDLE* phIface) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG2(hSystem, sIfaceID);
-	
+
     if(!verifyHandle(hSystem, Handle::TYPE_SYSTEM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<System*>(hSystem)->openInterface(sIfaceID, phIface));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API TLGetInfo(TL_HANDLE hSystem, TL_INFO_CMD iInfoCmd, INFO_DATATYPE* piType,
         void* pBuffer, size_t* piSize) {
-	GENTL_TRY;
+    GENTL_TRY;
     DEBUG2(hSystem, iInfoCmd);
-	
+
     if(!verifyHandle(hSystem, Handle::TYPE_SYSTEM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<System*>(hSystem)->getInfo(iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 
@@ -248,109 +260,109 @@ GC_API TLGetInfo(TL_HANDLE hSystem, TL_INFO_CMD iInfoCmd, INFO_DATATYPE* piType,
 
 GC_API IFClose(IF_HANDLE hIface) {
     GENTL_TRY;
-	DEBUG1(hIface);
-	
+    DEBUG1(hIface);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->close());
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API IFUpdateDeviceList(IF_HANDLE hIface, bool8_t* pbChanged, uint64_t iTimeout) {
     GENTL_TRY;
-	DEBUG2(hIface, iTimeout);
-	
+    DEBUG2(hIface, iTimeout);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->updateDeviceList(pbChanged, iTimeout));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API IFGetNumDevices(IF_HANDLE hIface, uint32_t* piNumDevices) {
     GENTL_TRY;
-	DEBUG1(hIface);
-	
+    DEBUG1(hIface);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->getNumDevices(piNumDevices));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API IFGetDeviceID(IF_HANDLE hIface, uint32_t iIndex, char* sDeviceID, size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hIface, iIndex);
-	
+    DEBUG2(hIface, iIndex);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->getDeviceID(iIndex, sDeviceID, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API IFGetDeviceInfo(IF_HANDLE hIface, const char* sDeviceID, DEVICE_INFO_CMD iInfoCmd,
         INFO_DATATYPE* piType, void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG3(hIface, sDeviceID, iInfoCmd);
-	
+    DEBUG3(hIface, sDeviceID, iInfoCmd);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->getDeviceInfo(sDeviceID, iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API IFGetParentTL(IF_HANDLE hIface, TL_HANDLE* phSystem) {
     GENTL_TRY;
-	DEBUG2(hIface, phSystem);
-	
+    DEBUG2(hIface, phSystem);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->getParentTL(phSystem));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API IFGetInfo(IF_HANDLE hIface, INTERFACE_INFO_CMD iInfoCmd, INFO_DATATYPE* piType,
         void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hIface, iInfoCmd);
-	
+    DEBUG2(hIface, iInfoCmd);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->getInfo(iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API IFOpenDevice(IF_HANDLE hIface, const char* sDeviceID, DEVICE_ACCESS_FLAGS iOpenFlags,
         DEV_HANDLE* phDevice) {
     GENTL_TRY;
-	DEBUG3(hIface, sDeviceID, iOpenFlags);
-	
+    DEBUG3(hIface, sDeviceID, iOpenFlags);
+
     if(!verifyHandle(hIface, Handle::TYPE_INTERFACE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Interface*>(hIface)->openDevice(sDeviceID, iOpenFlags, phDevice));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 
@@ -360,8 +372,8 @@ GC_API IFOpenDevice(IF_HANDLE hIface, const char* sDeviceID, DEVICE_ACCESS_FLAGS
 
 GC_API DevClose(DEV_HANDLE hDevice) {
     GENTL_TRY;
-	DEBUG1(hDevice);
-	
+    DEBUG1(hDevice);
+
     if(!verifyHandle(hDevice, Handle::TYPE_DEVICE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
@@ -369,90 +381,90 @@ GC_API DevClose(DEV_HANDLE hDevice) {
         reinterpret_cast<LogicalDevice*>(hDevice)->getPhysicalDevice()->getInterface()->getSystem()->freeUnusedDevices();
         return setLastError(error);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DevGetParentIF(DEV_HANDLE hDevice, IF_HANDLE* phIface) {
     GENTL_TRY;
-	DEBUG1(hDevice);
-	
+    DEBUG1(hDevice);
+
     if(!verifyHandle(hDevice, Handle::TYPE_DEVICE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<LogicalDevice*>(hDevice)->getParentIF(phIface));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DevGetInfo(DEV_HANDLE hDevice, DEVICE_INFO_CMD iInfoCmd, INFO_DATATYPE* piType,
         void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hDevice, iInfoCmd);
-	
+    DEBUG2(hDevice, iInfoCmd);
+
     if(!verifyHandle(hDevice, Handle::TYPE_DEVICE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<LogicalDevice*>(hDevice)->getInfo(
             iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DevGetNumDataStreams(DEV_HANDLE hDevice, uint32_t* piNumDataStreams) {
     GENTL_TRY;
-	DEBUG1(hDevice);
-	
+    DEBUG1(hDevice);
+
     if(!verifyHandle(hDevice, Handle::TYPE_DEVICE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<LogicalDevice*>(hDevice)->getNumDataStreams(piNumDataStreams));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DevGetPort(DEV_HANDLE hDevice, PORT_HANDLE* phRemoteDev) {
     GENTL_TRY;
-	DEBUG1(hDevice);
-	
+    DEBUG1(hDevice);
+
     if(!verifyHandle(hDevice, Handle::TYPE_DEVICE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<LogicalDevice*>(hDevice)->getPort(phRemoteDev));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DevGetDataStreamID(DEV_HANDLE hDevice, uint32_t iIndex, char* sDataStreamID,
         size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hDevice, iIndex);
-	
+    DEBUG2(hDevice, iIndex);
+
     if(!verifyHandle(hDevice, Handle::TYPE_DEVICE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<LogicalDevice*>(hDevice)->getDataStreamID(
             iIndex, sDataStreamID, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DevOpenDataStream(DEV_HANDLE hDevice, const char* sDataStreamID, DS_HANDLE* phDataStream) {
     GENTL_TRY;
-	DEBUG2(hDevice, sDataStreamID);
-	
+    DEBUG2(hDevice, sDataStreamID);
+
     if(!verifyHandle(hDevice, Handle::TYPE_DEVICE)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<LogicalDevice*>(hDevice)->openDataStream(sDataStreamID, phDataStream));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 
@@ -462,202 +474,202 @@ GC_API DevOpenDataStream(DEV_HANDLE hDevice, const char* sDataStreamID, DS_HANDL
 
 GC_API DSAnnounceBuffer(DS_HANDLE hDataStream, void* pBuffer, size_t iSize, void* pPrivate, BUFFER_HANDLE* phBuffer) {
     GENTL_TRY;
-	DEBUG4(hDataStream, pBuffer, iSize, pPrivate);
-	
+    DEBUG4(hDataStream, pBuffer, iSize, pPrivate);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->announceBuffer(pBuffer, iSize, pPrivate, phBuffer));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSAllocAndAnnounceBuffer(DS_HANDLE hDataStream, size_t iBufferSize, void* pPrivate, BUFFER_HANDLE* phBuffer) {
     GENTL_TRY;
-	DEBUG2(hDataStream, iBufferSize);
-	
+    DEBUG2(hDataStream, iBufferSize);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->allocAndAnnounceBuffer(iBufferSize, pPrivate, phBuffer));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSClose(DS_HANDLE hDataStream) {
     GENTL_TRY;
-	DEBUG1(hDataStream);
-	
+    DEBUG1(hDataStream);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->close());
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSRevokeBuffer(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, void ** ppBuffer, void ** ppPrivate) {
     GENTL_TRY;
-	DEBUG2(hDataStream, hBuffer);
-	
+    DEBUG2(hDataStream, hBuffer);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->revokeBuffer(hBuffer, ppBuffer, ppPrivate));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSQueueBuffer(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer) {
     GENTL_TRY;
-	DEBUG2(hDataStream, hBuffer);
-	
+    DEBUG2(hDataStream, hBuffer);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->queueBuffer(hBuffer));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSGetParentDev(DS_HANDLE hDataStream, DEV_HANDLE* phDevice) {
     GENTL_TRY;
-	DEBUG1(hDataStream);
-	
-	if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
+    DEBUG1(hDataStream);
+
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getParentDev(phDevice));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSStartAcquisition(DS_HANDLE hDataStream, ACQ_START_FLAGS iStartFlags, uint64_t iNumToAcquire) {
     GENTL_TRY;
-	DEBUG2(hDataStream, iNumToAcquire);
-	
+    DEBUG2(hDataStream, iNumToAcquire);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->startAcquisition(iStartFlags, iNumToAcquire));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSStopAcquisition(DS_HANDLE hDataStream, ACQ_STOP_FLAGS iStopFlags) {
     GENTL_TRY;
-	DEBUG1(hDataStream);
-	
+    DEBUG1(hDataStream);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->stopAcquisition(iStopFlags));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSFlushQueue(DS_HANDLE hDataStream, ACQ_QUEUE_TYPE iOperation) {
     GENTL_TRY;
-	DEBUG2(hDataStream, iOperation);
-	
+    DEBUG2(hDataStream, iOperation);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->flushQueue(iOperation));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSGetBufferChunkData(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, SINGLE_CHUNK_DATA* pChunkData,
         size_t* piNumChunks) {
     GENTL_TRY;
-	DEBUG2(hDataStream, hBuffer);
-	
+    DEBUG2(hDataStream, hBuffer);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getBufferChunkData(hBuffer, pChunkData, piNumChunks));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSGetBufferID(DS_HANDLE hDataStream, uint32_t iIndex, BUFFER_HANDLE* phBuffer) {
-	GENTL_TRY;
-	DEBUG2(hDataStream, iIndex);
-	
-	if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
+    GENTL_TRY;
+    DEBUG2(hDataStream, iIndex);
+
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getBufferID(iIndex, phBuffer));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSGetBufferInfo(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, BUFFER_INFO_CMD iInfoCmd, INFO_DATATYPE* piType,
         void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG3(hDataStream, hBuffer, iInfoCmd);
-	
+    DEBUG3(hDataStream, hBuffer, iInfoCmd);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getBufferInfo(hBuffer, iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSGetInfo(DS_HANDLE hDataStream, STREAM_INFO_CMD iInfoCmd, INFO_DATATYPE* piType,
         void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hDataStream, iInfoCmd);
-	
+    DEBUG2(hDataStream, iInfoCmd);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getInfo(iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSGetNumBufferParts(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, uint32_t *piNumParts ) {
     GENTL_TRY;
-	DEBUG2(hDataStream, hBuffer);
-	
+    DEBUG2(hDataStream, hBuffer);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getNumBufferParts(hBuffer, piNumParts));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API DSGetBufferPartInfo(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, uint32_t iPartIndex,
         BUFFER_PART_INFO_CMD iInfoCmd, INFO_DATATYPE *piType, void *pBuffer, size_t *piSize) {
     GENTL_TRY;
-	DEBUG4(hDataStream, hBuffer, iPartIndex, iInfoCmd);
-	
+    DEBUG4(hDataStream, hBuffer, iPartIndex, iInfoCmd);
+
     if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getBufferPartInfo(hBuffer, iPartIndex,
             iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 
@@ -668,8 +680,8 @@ GC_API DSGetBufferPartInfo(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, uint32_
 GC_API GCGetPortInfo(PORT_HANDLE hPort, PORT_INFO_CMD iInfoCmd, INFO_DATATYPE* piType,
         void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hPort, iInfoCmd);
-	
+    DEBUG2(hPort, iInfoCmd);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->getPortInfo(iInfoCmd, piType, pBuffer, piSize));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -683,14 +695,14 @@ GC_API GCGetPortInfo(PORT_HANDLE hPort, PORT_INFO_CMD iInfoCmd, INFO_DATATYPE* p
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCGetPortURL(PORT_HANDLE hPort, char* sURL, size_t* piSize) {
     GENTL_TRY;
-	DEBUG1(hPort);
-	
+    DEBUG1(hPort);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->getPortURL(sURL, piSize));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -704,14 +716,14 @@ GC_API GCGetPortURL(PORT_HANDLE hPort, char* sURL, size_t* piSize) {
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCGetNumPortURLs(PORT_HANDLE hPort, uint32_t* piNumURLs) {
     GENTL_TRY;
-	DEBUG1(hPort);
-	
+    DEBUG1(hPort);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->getNumPortURLs(piNumURLs));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -725,15 +737,15 @@ GC_API GCGetNumPortURLs(PORT_HANDLE hPort, uint32_t* piNumURLs) {
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCGetPortURLInfo(PORT_HANDLE hPort, uint32_t iURLIndex, URL_INFO_CMD iInfoCmd,
         INFO_DATATYPE* piType, void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG3(hPort, iURLIndex, iInfoCmd);
-	
+    DEBUG3(hPort, iURLIndex, iInfoCmd);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->getPortURLInfo(iURLIndex, iInfoCmd, piType, pBuffer, piSize));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -747,14 +759,14 @@ GC_API GCGetPortURLInfo(PORT_HANDLE hPort, uint32_t iURLIndex, URL_INFO_CMD iInf
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCReadPort(PORT_HANDLE hPort, uint64_t iAddress, void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hPort, iAddress);
-	
+    DEBUG2(hPort, iAddress);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->readPort(iAddress, pBuffer, piSize));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -768,14 +780,14 @@ GC_API GCReadPort(PORT_HANDLE hPort, uint64_t iAddress, void* pBuffer, size_t* p
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCWritePort(PORT_HANDLE hPort, uint64_t iAddress, const void* pBuffer, size_t* piSize) {
     GENTL_TRY;
-	DEBUG2(hPort, iAddress);
-	
+    DEBUG2(hPort, iAddress);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->writePort(iAddress, pBuffer, piSize));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -789,14 +801,14 @@ GC_API GCWritePort(PORT_HANDLE hPort, uint64_t iAddress, const void* pBuffer, si
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCWritePortStacked(PORT_HANDLE hPort, PORT_REGISTER_STACK_ENTRY* pEntries, size_t* piNumEntries) {
     GENTL_TRY;
-	DEBUG1(hPort);
-	
+    DEBUG1(hPort);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->writePortStacked(pEntries, piNumEntries));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -810,14 +822,14 @@ GC_API GCWritePortStacked(PORT_HANDLE hPort, PORT_REGISTER_STACK_ENTRY* pEntries
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCReadPortStacked(PORT_HANDLE hPort, PORT_REGISTER_STACK_ENTRY* pEntries, size_t* piNumEntries) {
     GENTL_TRY;
-	DEBUG1(hPort);
-	
+    DEBUG1(hPort);
+
     if(verifyHandle(hPort, Handle::TYPE_PORT)) {
         return setLastError(reinterpret_cast<Port*>(hPort)->readPortStacked(pEntries, piNumEntries));
     } else if(verifyHandle(hPort, Handle::TYPE_SYSTEM)) {
@@ -831,8 +843,8 @@ GC_API GCReadPortStacked(PORT_HANDLE hPort, PORT_REGISTER_STACK_ENTRY* pEntries,
     } else {
         return setLastError(GC_ERR_INVALID_HANDLE);
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 
@@ -842,96 +854,187 @@ GC_API GCReadPortStacked(PORT_HANDLE hPort, PORT_REGISTER_STACK_ENTRY* pEntries,
 
 GC_API EventFlush(EVENT_HANDLE hEvent) {
     GENTL_TRY;
-	DEBUG1(hEvent);
-	
+    DEBUG1(hEvent);
+
     if(!verifyHandle(hEvent, Handle::TYPE_EVENT)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Event*>(hEvent)->flush());
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API EventGetData(EVENT_HANDLE hEvent, void* pBuffer, size_t* piSize, uint64_t iTimeout) {
     GENTL_TRY;
-	DEBUG2(hEvent, iTimeout);
-	
+    DEBUG2(hEvent, iTimeout);
+
     if(!verifyHandle(hEvent, Handle::TYPE_EVENT)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Event*>(hEvent)->getData(pBuffer, piSize, iTimeout));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API EventGetDataInfo(EVENT_HANDLE hEvent, const void* pInBuffer, size_t iInSize,
         EVENT_DATA_INFO_CMD iInfoCmd, INFO_DATATYPE* piType, void* pOutBuffer, size_t* piOutSize) {
     GENTL_TRY;
-	DEBUG3(hEvent, iInSize, iInfoCmd);
-	
+    DEBUG3(hEvent, iInSize, iInfoCmd);
+
     if(!verifyHandle(hEvent, Handle::TYPE_EVENT)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Event*>(hEvent)->getDataInfo(pInBuffer, iInSize, iInfoCmd, piType, pOutBuffer, piOutSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API EventGetInfo(EVENT_HANDLE hEvent, EVENT_INFO_CMD iInfoCmd, INFO_DATATYPE * piType,
         void * pBuffer, size_t * piSize) {
     GENTL_TRY;
-	DEBUG2(hEvent, iInfoCmd);
-	
+    DEBUG2(hEvent, iInfoCmd);
+
     if(!verifyHandle(hEvent, Handle::TYPE_EVENT)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Event*>(hEvent)->getInfo(iInfoCmd, piType, pBuffer, piSize));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API EventKill(EVENT_HANDLE hEvent) {
     GENTL_TRY;
-	DEBUG1(hEvent);
-	
+    DEBUG1(hEvent);
+
     if(!verifyHandle(hEvent, Handle::TYPE_EVENT)) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(reinterpret_cast<Event*>(hEvent)->kill());
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCRegisterEvent(EVENTSRC_HANDLE hModule, EVENT_TYPE iEventID, EVENT_HANDLE* phEvent) {
     GENTL_TRY;
-	DEBUG2(hModule, iEventID);
-	
+    DEBUG2(hModule, iEventID);
+
     if(hModule == nullptr) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(Event::registerEvent(hModule, iEventID, phEvent));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
 GC_API GCUnregisterEvent(EVENTSRC_HANDLE hModule, EVENT_TYPE iEventID) {
     GENTL_TRY;
-	DEBUG2(hModule, iEventID);
-	
+    DEBUG2(hModule, iEventID);
+
     if (hModule == nullptr) {
         return setLastError(GC_ERR_INVALID_HANDLE);
     } else {
         return setLastError(Event::unregisterEvent(hModule, iEventID));
     }
-	
-	GENTL_CATCH;
+
+    GENTL_CATCH;
 }
 
+/**** GenTL 1.6 additions ****/
+
+GC_API DSAnnounceCompositeBuffer(DS_HANDLE hDataStream, size_t iNumSegments, void **ppSegments,
+        size_t *piSizes, void *pPrivate, BUFFER_HANDLE *phBuffer) {
+    GENTL_TRY;
+    DEBUG6(hDataStream,iNumSegments,ppSegments,piSizes,pPrivate,phBuffer);
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(phBuffer, Handle::TYPE_BUFFER)) {
+        return setLastError(GC_ERR_INVALID_HANDLE);
+    } else {
+        return setLastError(reinterpret_cast<DataStream*>(hDataStream)->announceCompositeBuffer(iNumSegments, ppSegments,
+            piSizes, pPrivate, phBuffer));
+    }
+    GENTL_CATCH;
 }
 
+GC_API DSGetBufferInfoStacked(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer,
+        DS_BUFFER_INFO_STACKED *pInfoStacked, size_t iNumInfos) {
+    GENTL_TRY;
+    DEBUG4(hDataStream,hBuffer,pInfoStacked,iNumInfos);
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
+        return setLastError(GC_ERR_INVALID_HANDLE);
+    } else {
+        return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getBufferInfoStacked(
+            hBuffer, pInfoStacked, iNumInfos));
+    }
+    GENTL_CATCH;
+}
+
+GC_API DSGetBufferPartInfoStacked(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer,
+        DS_BUFFER_PART_INFO_STACKED *pInfoStacked, size_t iNumInfos) {
+    GENTL_TRY;
+    DEBUG4(hDataStream,hBuffer,pInfoStacked,iNumInfos);
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
+        return setLastError(GC_ERR_INVALID_HANDLE);
+    } else {
+        return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getBufferPartInfoStacked(
+            hBuffer, pInfoStacked, iNumInfos));
+    }
+    GENTL_CATCH;
+}
+
+GC_API DSGetNumFlows(DS_HANDLE hDataStream, uint32_t *piNumFlows) {
+    GENTL_TRY;
+    DEBUG2(hDataStream,piNumFlows);
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
+        return setLastError(GC_ERR_INVALID_HANDLE);
+    } else {
+        return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getNumFlows(piNumFlows));
+    }
+    GENTL_CATCH;
+}
+
+GC_API DSGetFlowInfo(DS_HANDLE hDataStream, uint32_t iFlowIndex, FLOW_INFO_CMD iInfoCmd,
+        INFO_DATATYPE *piType, void *pBuffer, size_t *piSize) {
+    GENTL_TRY;
+    DEBUG6(hDataStream,iFlowIndex,iInfoCmd,piType,pBuffer,piSize);
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM)) {
+        return setLastError(GC_ERR_INVALID_HANDLE);
+    } else {
+        return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getFlowInfo(
+            iFlowIndex, iInfoCmd, piType, pBuffer, piSize));
+    }
+    GENTL_CATCH;
+}
+
+GC_API DSGetNumBufferSegments(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, uint32_t *piNumSegments) {
+    GENTL_TRY;
+    DEBUG3(hDataStream,hBuffer,piNumSegments);
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
+        return setLastError(GC_ERR_INVALID_HANDLE);
+    } else {
+        return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getNumBufferSegments(
+            hBuffer, piNumSegments));
+    }
+    GENTL_CATCH;
+}
+
+GC_API DSGetBufferSegmentInfo(DS_HANDLE hDataStream, BUFFER_HANDLE hBuffer, uint32_t iSegmentIndex,
+        SEGMENT_INFO_CMD iInfoCmd, INFO_DATATYPE *piType, void *pBuffer, size_t *piSize) {
+    GENTL_TRY;
+    DEBUG7(hDataStream,hBuffer,iSegmentIndex,iInfoCmd,piType,pBuffer,piSize);
+    if(!verifyHandle(hDataStream, Handle::TYPE_STREAM) || !verifyHandle(hBuffer, Handle::TYPE_BUFFER)) {
+        return setLastError(GC_ERR_INVALID_HANDLE);
+    } else {
+        return setLastError(reinterpret_cast<DataStream*>(hDataStream)->getBufferSegmentInfo(
+            hBuffer, iSegmentIndex, iInfoCmd, piType, pBuffer, piSize));
+    }
+    GENTL_CATCH;
+}
+
+
+} // namespace
+
