@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2020 Nerian Vision GmbH
+ * Copyright (c) 2022 Nerian Vision GmbH
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,7 +40,7 @@ public:
 
     // Redeclaration of public members
     void setRawTransferData(const ImageSet& metaData, const std::vector<unsigned char*>& rawData,
-        int firstTileWidth = 0, int secondTileWidth = 0, int validBytes = 0x7FFFFFFF);
+        int firstTileWidth = 0, int middleTileWidth = 0, int lastTileWidth = 0);
     void setRawValidBytes(const std::vector<int>& validBytes);
     void setTransferImageSet(const ImageSet& imageSet);
     TransferStatus transferData();
@@ -114,8 +114,8 @@ ImageTransfer::~ImageTransfer() {
 }
 
 void ImageTransfer::setRawTransferData(const ImageSet& metaData, const std::vector<unsigned char*>& rawData,
-        int firstTileWidth, int secondTileWidth, int validBytes) {
-    pimpl->setRawTransferData(metaData, rawData, firstTileWidth, secondTileWidth, validBytes);
+        int firstTileWidth, int middleTileWidth, int lastTileWidth) {
+    pimpl->setRawTransferData(metaData, rawData, firstTileWidth, middleTileWidth, lastTileWidth);
 }
 
 void ImageTransfer::setRawValidBytes(const std::vector<int>& validBytes) {
@@ -225,7 +225,7 @@ void ImageTransfer::Pimpl::initTcpServer(const addrinfo* addressInfo) {
     tcpServerSocket = ::socket(addressInfo->ai_family, addressInfo->ai_socktype,
         addressInfo->ai_protocol);
     if (tcpServerSocket == INVALID_SOCKET)  {
-        TransferException ex("Error opening socket: " + string(strerror(errno)));
+        TransferException ex("Error opening socket: " + Networking::getLastErrorString());
         throw ex;
     }
 
@@ -248,7 +248,7 @@ void ImageTransfer::Pimpl::initUdp(const addrinfo* addressInfo) {
     // Create sockets
     clientSocket = socket(AF_INET, SOCK_DGRAM, 0);
     if(clientSocket == INVALID_SOCKET) {
-        TransferException ex("Error creating receive socket: " + string(strerror(errno)));
+        TransferException ex("Error creating receive socket: " + Networking::getLastErrorString());
         throw ex;
     }
 
@@ -273,9 +273,6 @@ bool ImageTransfer::Pimpl::tryAccept() {
         throw TransferException("Connections can only be accepted in tcp server mode");
     }
 
-    unique_lock<recursive_mutex> recvLock(receiveMutex);
-    unique_lock<recursive_mutex> sendLock(sendMutex);
-
     // Accept one connection
     SOCKET newSocket = Networking::acceptConnection(tcpServerSocket, remoteAddress);
     if(newSocket == INVALID_SOCKET) {
@@ -283,6 +280,10 @@ bool ImageTransfer::Pimpl::tryAccept() {
         return false;
     }
 
+    // For a new connection we require locks
+    unique_lock<recursive_mutex> recvLock(receiveMutex);
+    unique_lock<recursive_mutex> sendLock(sendMutex);
+
     if(clientSocket != INVALID_SOCKET) {
         Networking::closeSocket(clientSocket);
     }
@@ -324,9 +325,9 @@ void ImageTransfer::Pimpl::setSocketOptions() {
 }
 
 void ImageTransfer::Pimpl::setRawTransferData(const ImageSet& metaData,
-        const std::vector<unsigned char*>& rawDataVec, int firstTileWidth, int secondTileWidth, int validBytes) {
+        const std::vector<unsigned char*>& rawDataVec, int firstTileWidth, int middleTileWidth, int lastTileWidth) {
     unique_lock<recursive_mutex> sendLock(sendMutex);
-    protocol->setRawTransferData(metaData, rawDataVec, firstTileWidth, secondTileWidth, validBytes);
+    protocol->setRawTransferData(metaData, rawDataVec, firstTileWidth, middleTileWidth, lastTileWidth);
     currentMsg = nullptr;
 }
 
@@ -353,6 +354,14 @@ ImageTransfer::TransferStatus ImageTransfer::Pimpl::transferData() {
         return NOT_CONNECTED;
     }
 
+#ifndef _WIN32
+    // Cork TCP to prevent sending of small packets
+    if(protType == ImageProtocol::PROTOCOL_TCP) {
+        int flag = 1;
+        setsockopt(clientSocket, IPPROTO_TCP, TCP_CORK, (char *) &flag, sizeof(int));
+    }
+#endif
+
     // Get first message to transfer
     if(currentMsg == nullptr) {
         currentMsgOffset = 0;
@@ -368,6 +377,7 @@ ImageTransfer::TransferStatus ImageTransfer::Pimpl::transferData() {
     }
 
     // Try transferring messages
+    bool wouldBlock = false;
     bool dataTransferred = (currentMsg != nullptr);
     while(currentMsg != nullptr) {
         int writing = (int)(currentMsgLen - currentMsgOffset);
@@ -377,16 +387,24 @@ ImageTransfer::TransferStatus ImageTransfer::Pimpl::transferData() {
             currentMsgOffset = 0;
             currentMsg = protocol->getTransferMessage(currentMsgLen);
         } else {
-            return WOULD_BLOCK;
+            // The operation would block
+            wouldBlock = true;
+            break;
         }
     }
 
     if(dataTransferred && protType == ImageProtocol::PROTOCOL_TCP && protocol->transferComplete()) {
+#ifndef _WIN32
+        // Uncork - sends the assembled messages
+        int flag = 0;
+        setsockopt(clientSocket, IPPROTO_TCP, TCP_CORK, (char *) &flag, sizeof(int));
+#else
         // Force a flush for TCP by turning the nagle algorithm off and on
         int flag = 1;
         setsockopt(clientSocket, IPPROTO_TCP, TCP_NODELAY, (char *) &flag, sizeof(int));
         flag = 0;
         setsockopt(clientSocket, IPPROTO_TCP, TCP_NODELAY, (char *) &flag, sizeof(int));
+#endif
     }
 
     // Also check for control messages at the end
@@ -396,6 +414,8 @@ ImageTransfer::TransferStatus ImageTransfer::Pimpl::transferData() {
 
     if(protocol->transferComplete()) {
         return ALL_TRANSFERRED;
+    } else if(wouldBlock) {
+        return WOULD_BLOCK;
     } else {
         return PARTIAL_TRANSFER;
     }
@@ -413,7 +433,7 @@ bool ImageTransfer::Pimpl::receiveImageSet(ImageSet& imageSet) {
 
         unsigned int time = static_cast<unsigned int>(std::chrono::duration_cast<std::chrono::milliseconds>(
             std::chrono::steady_clock::now() - startTime).count());
-        if(time > 1000) {
+        if(time > 100 && !complete) {
             return false;
         }
     }
@@ -453,7 +473,7 @@ bool ImageTransfer::Pimpl::receiveNetworkData(bool block) {
 
     // Test if the socket has data available
     if(!block && !selectSocket(true, false)) {
-        return 0;
+        return false;
     }
 
     int maxLength = 0;
@@ -466,17 +486,16 @@ bool ImageTransfer::Pimpl::receiveNetworkData(bool block) {
     int bytesReceived = recvfrom(clientSocket, buffer, maxLength,
         0, reinterpret_cast<sockaddr*>(&fromAddress), &fromSize);
 
-    //std::cout << "recvfrom to addr " << ((long) buffer) << std::endl;
-    //std::cout << "recvfrom returned " << bytesReceived << std::endl;
-    if(bytesReceived == 0 || (protType == ImageProtocol::PROTOCOL_TCP && bytesReceived < 0 && errno == WSAECONNRESET)) {
+    auto err = Networking::getErrno();
+    if(bytesReceived == 0 || (protType == ImageProtocol::PROTOCOL_TCP && bytesReceived < 0 && err == WSAECONNRESET)) {
         // Connection closed
         disconnect();
-    } else if(bytesReceived < 0 && errno != EWOULDBLOCK && errno != EINTR &&
-            errno != ETIMEDOUT && errno != WSA_IO_PENDING && errno != WSAECONNRESET) {
-        TransferException ex("Error reading from socket: " + string(strerror(errno)));
+    } else if(bytesReceived < 0 && err != EWOULDBLOCK && err != EINTR &&
+            err != ETIMEDOUT && err != WSA_IO_PENDING && err != WSAECONNRESET) {
+        TransferException ex("Error reading from socket: " + Networking::getErrorString(err));
         throw ex;
     } else if(bytesReceived > 0) {
-            protocol->processReceivedMessage(bytesReceived);
+        protocol->processReceivedMessage(bytesReceived);
         if(protocol->newClientConnected()) {
             // We have just established a new connection
             memcpy(&remoteAddress, &fromAddress, sizeof(remoteAddress));
@@ -530,7 +549,7 @@ bool ImageTransfer::Pimpl::sendNetworkMessage(const unsigned char* msg, int leng
         written = send(destSocket, reinterpret_cast<const char*>(msg), length, 0);
     }
 
-    unsigned long sendError = errno;
+    auto sendError = Networking::getErrno();
 
     if(written < 0) {
         if(sendError == EAGAIN || sendError == EWOULDBLOCK || sendError == ETIMEDOUT) {
@@ -541,7 +560,7 @@ bool ImageTransfer::Pimpl::sendNetworkMessage(const unsigned char* msg, int leng
             disconnect();
             return false;
         } else {
-            TransferException ex("Error sending network packet: " + string(strerror(sendError)));
+            TransferException ex("Error sending network packet: " + Networking::getErrorString(sendError));
             throw ex;
         }
     } else if(written != length) {
@@ -571,7 +590,6 @@ void ImageTransfer::Pimpl::sendPendingControlMessages() {
         controlMsgData = protocol->getNextControlMessage(controlMsgLen);
 
         if(controlMsgData != nullptr) {
-            //std::cout << "Sending a control message of size " << controlMsgLen << std::endl;
             sendNetworkMessage(controlMsgData, controlMsgLen);
         } else {
             break;
