@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2020 Nerian Vision GmbH
+ * Copyright (c) 2022 Nerian Vision GmbH
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,17 +41,20 @@ public:
     Pimpl();
 
     float* createPointMap(const unsigned short* dispMap, int width, int height,
-        int rowStride, const float* q, unsigned short minDisparity);
+        int rowStride, const float* q, unsigned short minDisparity, int subpixelFactor,
+        unsigned short maxDisparity);
 
     float* createPointMap(const ImageSet& imageSet, unsigned short minDisparity);
 
+    float* createZMap(const ImageSet& imageSet, unsigned short minDisparity, unsigned short maxDisparity);
+
     void projectSinglePoint(int imageX, int imageY, unsigned short disparity, const float* q,
-        float& pointX, float& pointY, float& pointZ);
+        float& pointX, float& pointY, float& pointZ, int subpixelFactor);
 
     void writePlyFile(const char* file, const unsigned short* dispMap,
         const unsigned char* image, int width, int height, ImageSet::ImageFormat format,
         int dispRowStride, int imageRowStride, const float* q,
-        double maxZ, bool binary);
+        double maxZ, bool binary, int subpixelFactor, unsigned short maxDisparity);
 
    void writePlyFile(const char* file, const ImageSet& imageSet,
         double maxZ, bool binary);
@@ -60,13 +63,16 @@ private:
     std::vector<float, AlignedAllocator<float> > pointMap;
 
     float* createPointMapFallback(const unsigned short* dispMap, int width, int height,
-        int rowStride, const float* q, unsigned short minDisparity);
+        int rowStride, const float* q, unsigned short minDisparity, int subpixelFactor,
+        unsigned short maxDisparity);
 
     float* createPointMapSSE2(const unsigned short* dispMap, int width, int height,
-        int rowStride, const float* q, unsigned short minDisparity);
+        int rowStride, const float* q, unsigned short minDisparity, int subpixelFactor,
+        unsigned short maxDisparity);
 
     float* createPointMapAVX2(const unsigned short* dispMap, int width, int height,
-        int rowStride, const float* q, unsigned short minDisparity);
+        int rowStride, const float* q, unsigned short minDisparity, int subpixelFactor,
+        unsigned short maxDisparity);
 };
 
 /******************** Stubs for all public members ********************/
@@ -80,24 +86,33 @@ Reconstruct3D::~Reconstruct3D() {
 }
 
 float* Reconstruct3D::createPointMap(const unsigned short* dispMap, int width, int height,
-        int rowStride, const float* q, unsigned short minDisparity) {
-    return pimpl->createPointMap(dispMap, width, height, rowStride, q, minDisparity);
+        int rowStride, const float* q, unsigned short minDisparity, int subpixelFactor,
+        unsigned short maxDisparity) {
+    return pimpl->createPointMap(dispMap, width, height, rowStride, q, minDisparity,
+        subpixelFactor, maxDisparity);
 }
 
 float* Reconstruct3D::createPointMap(const ImageSet& imageSet, unsigned short minDisparity) {
     return pimpl->createPointMap(imageSet, minDisparity);
 }
 
+float* Reconstruct3D::createZMap(const ImageSet& imageSet, unsigned short minDisparity,
+    unsigned short maxDisparity) {
+    return pimpl->createZMap(imageSet, minDisparity, maxDisparity);
+}
+
 void Reconstruct3D::projectSinglePoint(int imageX, int imageY, unsigned short disparity,
-        const float* q, float& pointX, float& pointY, float& pointZ) {
-    pimpl->projectSinglePoint(imageX, imageY, disparity, q, pointX, pointY, pointZ);
+        const float* q, float& pointX, float& pointY, float& pointZ, int subpixelFactor) {
+    pimpl->projectSinglePoint(imageX, imageY, disparity, q, pointX, pointY, pointZ,
+        subpixelFactor);
 }
 
 void Reconstruct3D::writePlyFile(const char* file, const unsigned short* dispMap,
         const unsigned char* image,  int width, int height, ImageSet::ImageFormat format, int dispRowStride,
-        int imageRowStride, const float* q, double maxZ, bool binary) {
+        int imageRowStride, const float* q, double maxZ, bool binary, int subpixelFactor,
+        unsigned short maxDisparity) {
     pimpl->writePlyFile(file, dispMap, image, width, height, format, dispRowStride,
-        imageRowStride, q, maxZ, binary);
+        imageRowStride, q, maxZ, binary, subpixelFactor, maxDisparity);
 }
 
 void Reconstruct3D::writePlyFile(const char* file, const ImageSet& imageSet,
@@ -111,24 +126,28 @@ Reconstruct3D::Pimpl::Pimpl() {
 }
 
 float* Reconstruct3D::Pimpl::createPointMap(const unsigned short* dispMap, int width,
-        int height, int rowStride, const float* q, unsigned short minDisparity) {
+        int height, int rowStride, const float* q, unsigned short minDisparity,
+        int subpixelFactor, unsigned short maxDisparity) {
 
     // Allocate the buffer
-    if(pointMap.size() != static_cast<unsigned int>(4*width*height)) {
+    if(pointMap.size() < static_cast<unsigned int>(4*width*height)) {
         pointMap.resize(4*width*height);
     }
 
 #   ifdef __AVX2__
-        if(width % 16 == 0) {
-            return createPointMapAVX2(dispMap, width, height, rowStride, q, minDisparity);
+        if(width % 16 == 0 && (uintptr_t)dispMap % 32 == 0) {
+            return createPointMapAVX2(dispMap, width, height, rowStride, q,
+                minDisparity, subpixelFactor, maxDisparity);
         } else
 #   endif
 #   ifdef __SSE2__
-        if(width % 8 == 0) {
-            return createPointMapSSE2(dispMap, width, height, rowStride, q, minDisparity);
+        if(width % 8 == 0 && (uintptr_t)dispMap % 16 == 0) {
+            return createPointMapSSE2(dispMap, width, height, rowStride, q,
+                minDisparity, subpixelFactor, maxDisparity);
         } else
 #   endif
-        return createPointMapFallback(dispMap, width, height, rowStride, q, minDisparity);
+        return createPointMapFallback(dispMap, width, height, rowStride, q,
+            minDisparity, subpixelFactor, maxDisparity);
 }
 
 float* Reconstruct3D::Pimpl::createPointMap(const ImageSet& imageSet, unsigned short minDisparity) {
@@ -141,11 +160,13 @@ float* Reconstruct3D::Pimpl::createPointMap(const ImageSet& imageSet, unsigned s
     }
 
     return createPointMap(reinterpret_cast<unsigned short*>(imageSet.getPixelData(ImageSet::IMAGE_DISPARITY)), imageSet.getWidth(),
-        imageSet.getHeight(), imageSet.getRowStride(ImageSet::IMAGE_DISPARITY), imageSet.getQMatrix(), minDisparity);
+        imageSet.getHeight(), imageSet.getRowStride(ImageSet::IMAGE_DISPARITY), imageSet.getQMatrix(), minDisparity,
+        imageSet.getSubpixelFactor(), 0xFFF);
 }
 
 float* Reconstruct3D::Pimpl::createPointMapFallback(const unsigned short* dispMap, int width,
-        int height, int rowStride, const float* q, unsigned short minDisparity) {
+        int height, int rowStride, const float* q, unsigned short minDisparity,
+        int subpixelFactor, unsigned short maxDisparity) {
     // Code without SSE or AVX optimization
     float* outputPtr = &pointMap[0];
     int stride = rowStride / 2;
@@ -156,13 +177,14 @@ float* Reconstruct3D::Pimpl::createPointMapFallback(const unsigned short* dispMa
         double qz = q[9]*y + q[11];
         double qw = q[13]*y + q[15];
 
+        const unsigned short* dispRow = &dispMap[y*stride];
         for(int x = 0; x < width; x++) {
-            unsigned short intDisp = std::max(minDisparity, dispMap[y*stride + x]);
-            if(intDisp >= 0xFFF) {
+            unsigned short intDisp = std::max(minDisparity, dispRow[x]);
+            if(intDisp >= maxDisparity) {
                 intDisp = minDisparity; // Invalid disparity
             }
 
-            double d = intDisp / 16.0;
+            double d = intDisp / double(subpixelFactor);
             double w = qw + q[14]*d;
 
             *outputPtr = static_cast<float>((qx + q[2]*d)/w); // x
@@ -183,10 +205,46 @@ float* Reconstruct3D::Pimpl::createPointMapFallback(const unsigned short* dispMa
     return &pointMap[0];
 }
 
-void Reconstruct3D::Pimpl::projectSinglePoint(int imageX, int imageY, unsigned short disparity,
-        const float* q, float& pointX, float& pointY, float& pointZ) {
+float* Reconstruct3D::Pimpl::createZMap(const ImageSet& imageSet, unsigned short minDisparity,
+        unsigned short maxDisparity) {
+    // Allocate the buffer
+    if(pointMap.size() < static_cast<unsigned int>(imageSet.getWidth()*imageSet.getHeight())) {
+        pointMap.resize(imageSet.getWidth()*imageSet.getHeight());
+    }
 
-    double d = disparity / 16.0;
+    float* outputPtr = &pointMap[0];
+    int stride = imageSet.getRowStride(ImageSet::IMAGE_DISPARITY) / 2;
+    const unsigned short* dispMap = reinterpret_cast<const unsigned short*>(imageSet.getPixelData(ImageSet::IMAGE_DISPARITY));
+    int subpixelFactor = imageSet.getSubpixelFactor();
+    const float* q = imageSet.getQMatrix();
+
+    for(int y = 0; y < imageSet.getHeight(); y++) {
+        double qz = q[9]*y + q[11];
+        double qw = q[13]*y + q[15];
+
+        const unsigned short* dispRow = &dispMap[y*stride];
+        for(int x = 0; x < imageSet.getWidth(); x++) {
+            unsigned short intDisp = std::max(minDisparity, dispRow[x]);
+            if(intDisp >= maxDisparity) {
+                intDisp = minDisparity; // Invalid disparity
+            }
+
+            double d = intDisp / double(subpixelFactor);
+            double w = qw + q[14]*d;
+
+            *outputPtr = static_cast<float>((qz + q[10]*d)/w); // z
+            outputPtr++;
+
+            qz += q[8];
+        }
+    }
+    return &pointMap[0];
+}
+
+void Reconstruct3D::Pimpl::projectSinglePoint(int imageX, int imageY, unsigned short disparity,
+        const float* q, float& pointX, float& pointY, float& pointZ, int subpixelFactor) {
+
+    double d = disparity / double(subpixelFactor);
     double w = q[15] + q[14]*d;
     pointX = static_cast<float>((imageX*q[0] + q[3])/w);
     pointY = static_cast<float>((imageY*q[5] + q[7])/w);
@@ -195,7 +253,8 @@ void Reconstruct3D::Pimpl::projectSinglePoint(int imageX, int imageY, unsigned s
 
 # ifdef __AVX2__
 float* Reconstruct3D::Pimpl::createPointMapAVX2(const unsigned short* dispMap, int width,
-        int height, int rowStride, const float* q, unsigned short minDisparity) {
+        int height, int rowStride, const float* q, unsigned short minDisparity,
+        int subpixelFactor, unsigned short maxDisparity) {
 
     // Create column vectors of q
     const __m256 qCol0 = _mm256_setr_ps(q[0], q[4], q[8], q[12],   q[0], q[4], q[8], q[12]);
@@ -205,8 +264,8 @@ float* Reconstruct3D::Pimpl::createPointMapAVX2(const unsigned short* dispMap, i
 
     // More constants that we need
     const __m256i minDispVector = _mm256_set1_epi16(minDisparity);
-    const __m256i maxDispVector = _mm256_set1_epi16(0xFFF);
-    const __m256 scaleVector = _mm256_set1_ps(1.0/16.0);
+    const __m256i maxDispVector = _mm256_set1_epi16(maxDisparity);
+    const __m256 scaleVector = _mm256_set1_ps(1.0/double(subpixelFactor));
     const __m256i zeroVector = _mm256_set1_epi16(0);
 
     float* outputPtr = &pointMap[0];
@@ -229,7 +288,7 @@ float* Reconstruct3D::Pimpl::createPointMapAVX2(const unsigned short* dispMap, i
             // Stupid AVX2 unpack mixes everything up! Lets swap the register beforehand.
             __m256i disparitiesMixup = _mm256_permute4x64_epi64(disparities, 0xd8);
 
-            // Convert to floats and scale with 1/16
+            // Convert to floats and scale with 1/subpixelFactor
             __m256 floatDisp = _mm256_cvtepi32_ps(_mm256_unpacklo_epi16(disparitiesMixup, zeroVector));
             __m256 dispScaled = _mm256_mul_ps(floatDisp, scaleVector);
 
@@ -284,7 +343,8 @@ float* Reconstruct3D::Pimpl::createPointMapAVX2(const unsigned short* dispMap, i
 
 #ifdef __SSE2__
 float* Reconstruct3D::Pimpl::createPointMapSSE2(const unsigned short* dispMap, int width,
-        int height, int rowStride, const float* q, unsigned short minDisparity) {
+        int height, int rowStride, const float* q, unsigned short minDisparity,
+        int subpixelFactor, unsigned short maxDisparity) {
 
     // Create column vectors of q
     const __m128 qCol0 = _mm_setr_ps(q[0], q[4], q[8], q[12]);
@@ -294,8 +354,8 @@ float* Reconstruct3D::Pimpl::createPointMapSSE2(const unsigned short* dispMap, i
 
     // More constants that we need
     const __m128i minDispVector = _mm_set1_epi16(minDisparity);
-    const __m128i maxDispVector = _mm_set1_epi16(0xFFF);
-    const __m128 scaleVector = _mm_set1_ps(1.0/16.0);
+    const __m128i maxDispVector = _mm_set1_epi16(maxDisparity);
+    const __m128 scaleVector = _mm_set1_ps(1.0/double(subpixelFactor));
     const __m128i zeroVector = _mm_set1_epi16(0);
 
     float* outputPtr = &pointMap[0];
@@ -315,7 +375,7 @@ float* Reconstruct3D::Pimpl::createPointMapSSE2(const unsigned short* dispMap, i
             // Clamp to minimum disparity
             disparities = _mm_max_epi16(disparities, minDispVector);
 
-            // Convert to floats and scale with 1/16
+            // Convert to floats and scale with 1/subpixelFactor
             __m128 floatDisp = _mm_cvtepi32_ps(_mm_unpacklo_epi16(disparities, zeroVector));
             __m128 dispScaled = _mm_mul_ps(floatDisp, scaleVector);
 
@@ -369,9 +429,11 @@ float* Reconstruct3D::Pimpl::createPointMapSSE2(const unsigned short* dispMap, i
 
 void Reconstruct3D::Pimpl::writePlyFile(const char* file, const unsigned short* dispMap,
         const unsigned char* image, int width, int height, ImageSet::ImageFormat format, int dispRowStride,
-        int imageRowStride, const float* q, double maxZ, bool binary) {
+        int imageRowStride, const float* q, double maxZ, bool binary, int subpixelFactor,
+        unsigned short maxDisparity) {
 
-    float* pointMap = createPointMap(dispMap, width, height, dispRowStride, q, 0);
+    float* pointMap = createPointMap(dispMap, width, height, dispRowStride,
+        q, 0, subpixelFactor, maxDisparity);
 
     // Count number of valid points
     int pointsCount = 0;
@@ -487,7 +549,7 @@ void Reconstruct3D::Pimpl::writePlyFile(const char* file, const ImageSet& imageS
         imageSet.getRowStride(indexDisp),
         (indexImg == -1) ? 0 : imageSet.getRowStride(indexImg),
         imageSet.getQMatrix(),
-        maxZ, binary);
+        maxZ, binary, imageSet.getSubpixelFactor(), 0xFFF);
 }
 
 } // namespace
